@@ -7,6 +7,7 @@ from subprocess import check_call, check_output
 import glob
 import logging
 import os
+import platform
 import shutil
 import stat
 import sys
@@ -19,7 +20,6 @@ import yaml
 from ..embedded.stacker.awscli_yamlhelper import yaml_parse as parse_cloudformation_template  # noqa
 from .. import __version__ as version
 
-logging.basicConfig(level=logging.INFO)
 LOGGER = logging.getLogger('runway')
 EMBEDDED_LIB_PATH = os.path.join(
     os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
@@ -31,7 +31,7 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
     """Base class for deployer classes."""
 
     def __init__(self, options, env_vars=None, env_root=None,  # noqa pylint: disable=too-many-arguments
-                 module_root=None, runway_config_dir=None):
+                 deploy_opts=None, module_root=None, runway_config_dir=None):
         """Initialize base class."""
         self.options = options
 
@@ -47,6 +47,11 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
 
         self.module_root = module_root
 
+        if deploy_opts is None:
+            self.deploy_opts = {}
+        else:
+            self.deploy_opts = deploy_opts
+
         if runway_config_dir is None:
             self.runway_config_path = os.path.join(
                 self.env_root,
@@ -62,6 +67,8 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
         self.environment_override_name = 'DEPLOY_ENVIRONMENT'
         self._environment_name = None
 
+        self.embedded_lib_path = EMBEDDED_LIB_PATH
+
     def update_env_vars(self, val):
         """Update env_vars dict with provided dict."""
         for key, value in list(val.items()):
@@ -98,6 +105,18 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
         if dirs_to_scan is None:
             dirs_to_scan = self.get_env_dirs()
 
+        if os.path.isfile(os.path.join(base_dir, '.flake8')):
+            # config file in env will be picked up automatically
+            flake8_config = []
+        else:
+            # no config file in env; use runway defaults
+            flake8_config = [
+                ('--append-config=' + os.path.join(
+                    os.path.dirname(os.path.dirname(os.path.abspath(__file__))),  # noqa
+                    'templates',
+                    '.flake8'
+                ))
+            ]
         if os.path.isfile(os.path.join(base_dir, '.yamllint.yml')):
             yamllint_config = os.path.join(base_dir, '.yamllint.yml')
         else:
@@ -107,13 +126,15 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
                 '.yamllint.yml'
             )
         with self.change_dir(base_dir):
-            LOGGER.info('Starting Flake8 linting...')
-            flake8_run = flake8_app.Application()
-            flake8_run.run(
-                ['--exclude=node_modules,.serverless'] + dirs_to_scan +  self.get_python_files_at_env_root()  # noqa pylint: disable=line-too-long
-            )
-            LOGGER.info('Flake8 linting complete.')
             with self.ignore_exit_code_0():
+                LOGGER.info('Starting Flake8 linting...')
+                flake8_run = flake8_app.Application()
+                flake8_run.run(
+                    flake8_config + dirs_to_scan +  self.get_python_files_at_env_root()  # noqa pylint: disable=line-too-long
+                )
+                flake8_run.exit()
+            with self.ignore_exit_code_0():
+                LOGGER.info('Flake8 linting complete.')
                 LOGGER.info('Starting yamllint...')
                 yamllint_run(
                     ["--config-file=%s" % yamllint_config] + dirs_to_scan + self.get_yaml_files_at_env_root()  # noqa pylint: disable=line-too-long
@@ -161,53 +182,64 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
             base_dir = self.env_root
         if pylint_rc_file is None:
             if os.path.isfile(os.path.join(base_dir, '.pylintrc')):
-                pylint_rc_config = [
+                pylint_config = [
                     "--rcfile=%s" % os.path.join(base_dir, '.pylintrc')
                 ]
             else:
-                pylint_rc_config = []
+                # Only reporting on errors ('-E') overrides any ignored errors
+                # set in .pylintrc, so it is only being used here when a
+                # pylint configuration file is not being used.
+                pylint_config = ['-E']
 
         # Check all python files in repo
         dirs_to_skip = set(['.git',
                             'node_modules',
                             '.serverless'])
-        for root, dirs, files in os.walk(base_dir):  # noqa pylint: disable=too-many-nested-blocks
+        nonblueprint_files = []
+        blueprint_files = []
+        for root, dirs, files in os.walk(base_dir):
             dirs[:] = [d for d in dirs if d not in dirs_to_skip]
             for name in files:
-                if name[-3:] == ".py":
-                    filepath = os.path.join(root, name)
-                    LOGGER.info("Checking %s with pylint (\"No config file "
-                                "found...\" messages can be ignored)",
-                                filepath)
-                    with self.use_embedded_pkgs():  # for embedded stacker
-                        with self.ignore_exit_code_0():
-                            PylintRun(pylint_rc_config + ['-E',  # ignore warn
-                                                          filepath])
-                    # Blueprints should output their template when executed
-                    if (root.endswith('blueprints') and
-                            not filepath.endswith('__init__.py')):
-                        self.ensure_file_is_executable(filepath)
-                        try:
-                            shell_out_env = os.environ.copy()
-                            if 'PYTHONPATH' in shell_out_env:
-                                shell_out_env['PYTHONPATH'] = (
-                                    "%s:%s" % (EMBEDDED_LIB_PATH,
-                                               shell_out_env['PYTHONPATH'])
-                                )
-                            else:
-                                shell_out_env['PYTHONPATH'] = EMBEDDED_LIB_PATH
-                            cfn_template = check_output(
-                                [sys.executable, filepath],
-                                env=shell_out_env
-                            )
-                            if cfn_template == '':
-                                raise ValueError("Template output should not "
-                                                 "be empty!")
-                            parse_cloudformation_template(cfn_template)
-                        except:  # noqa - Bare except fine in this context
-                            print("Error while checking %s for valid "
-                                  "YAML/JSON output" % filepath)
-                            raise
+                filepath = os.path.join(root, name)
+                if name[-3:] == '.py' and (
+                        root.endswith('blueprints') and
+                        not filepath.endswith('__init__.py')):
+                    blueprint_files.append(filepath)
+                elif name[-3:] == '.py':
+                    nonblueprint_files.append(filepath)
+
+        if nonblueprint_files + blueprint_files:
+            LOGGER.info("Checking python files with pylint (\"No config file "
+                        "found...\" messages can be ignored)")
+            with self.use_embedded_pkgs():  # for embedded stacker
+                with self.ignore_exit_code_0():
+                    LOGGER.debug("Executing pylint with the following options: \"%s\"",  # noqa
+                                 ' '.join(pylint_config + nonblueprint_files + blueprint_files))  # noqa pylint: disable=line-too-long
+                    PylintRun(pylint_config + nonblueprint_files + blueprint_files)  # noqa
+            LOGGER.info('pylint complete.')
+            for filepath in blueprint_files:
+                # Blueprints should output their template when executed
+                self.ensure_file_is_executable(filepath)
+                try:
+                    shell_out_env = os.environ.copy()
+                    if 'PYTHONPATH' in shell_out_env:
+                        shell_out_env['PYTHONPATH'] = (
+                            "%s:%s" % (self.embedded_lib_path,
+                                       shell_out_env['PYTHONPATH'])
+                        )
+                    else:
+                        shell_out_env['PYTHONPATH'] = self.embedded_lib_path
+                    cfn_template = check_output(
+                        [sys.executable, filepath],
+                        env=shell_out_env
+                    )
+                    if cfn_template == '':
+                        raise ValueError('Template output should not be empty!')  # noqa
+                    parse_cloudformation_template(cfn_template)
+                except:  # noqa - Bare except fine in this context
+                    print("Error while checking %s for valid "
+                          "YAML/JSON output" % filepath)
+                    raise
 
     def test(self):
         """Execute tests."""
@@ -231,17 +263,19 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
             elif i in self.env_vars:
                 self.env_vars.pop(i)
 
-    def assume_role(self, role_arn, session_name=None, region='us-east-1'):
+    def assume_role(self, role_arn, session_name=None, duration_seconds=None,
+                    region='us-east-1'):
         """Assume IAM role."""
         import boto3
         if session_name is None:
             session_name = 'runway'
+        assume_role_opts = {'RoleArn': role_arn,
+                            'RoleSessionName': session_name}
+        if duration_seconds:
+            assume_role_opts['DurationSeconds'] = int(duration_seconds)
         sts_client = boto3.client('sts', region_name=region)
         LOGGER.info("Assuming role %s...", role_arn)
-        response = sts_client.assume_role(
-            RoleArn=role_arn,
-            RoleSessionName=session_name
-        )
+        response = sts_client.assume_role(**assume_role_opts)
         self.update_env_vars(
             {'AWS_ACCESS_KEY_ID': response['Credentials']['AccessKeyId'],
              'AWS_SECRET_ACCESS_KEY': response['Credentials']['SecretAccessKey'],  # noqa
@@ -255,8 +289,14 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
                 self.save_existing_iam_env_vars()
             if assume_role_config.get('arn'):
                 assume_role_arn = assume_role_config['arn']
+                assume_role_duration = assume_role_config.get('duration')
             elif assume_role_config.get(self.environment_name):
-                assume_role_arn = assume_role_config[self.environment_name]
+                if isinstance(assume_role_config[self.environment_name], dict):
+                    assume_role_arn = assume_role_config[self.environment_name]['arn']  # noqa
+                    assume_role_duration = assume_role_config[self.environment_name].get('duration')  # noqa pylint: disable=line-too-long
+                else:
+                    assume_role_arn = assume_role_config[self.environment_name]
+                    assume_role_duration = None
             else:
                 LOGGER.info('Skipping assume-role; no role found for '
                             'environment %s...',
@@ -266,6 +306,7 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
             self.assume_role(
                 role_arn=assume_role_arn,
                 session_name=assume_role_config.get('session_name', None),
+                duration_seconds=assume_role_duration,
                 region=region
             )
             return True
@@ -381,8 +422,8 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
         if module_dir is None:
             module_dir = os.path.join(self.env_root, 'sampleapp.tf')
         self.generate_sample_module(module_dir)
-        for i in ['backend-us-east-1.tfvars', 'dev-us-east-1.tfvars',
-                  'main.tf']:
+        for i in ['.terraform-version', 'backend-us-east-1.tfvars',
+                  'dev-us-east-1.tfvars', 'main.tf']:
             shutil.copyfile(
                 os.path.join(os.path.dirname(os.path.dirname(__file__)),
                              'templates',
@@ -426,14 +467,13 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
 
     def parse_runway_config(self):
         """Read and parse runway.yml."""
-        if os.path.isfile(self.runway_config_path):
-            with open(self.runway_config_path) as data_file:
-                return yaml.safe_load(data_file)
-        else:
+        if not os.path.isfile(self.runway_config_path):
             LOGGER.error("Runway config file was not found (looking for "
                          "%s)",
                          self.runway_config_path)
             sys.exit(1)
+        with open(self.runway_config_path) as data_file:
+            return yaml.safe_load(data_file)
 
     def execute(self):
         """Implement dummy method (set in consuming classes)."""
@@ -454,6 +494,19 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
             self._runway_config = self.parse_runway_config()
         return self._runway_config
 
+    @contextmanager
+    def use_embedded_pkgs(self):
+        """Temporarily prepend embedded packages to sys.path."""
+        old_sys_path = list(sys.path)
+        sys.path.insert(
+            1,  # https://stackoverflow.com/a/10097543
+            self.embedded_lib_path
+        )
+        try:
+            yield
+        finally:
+            sys.path = old_sys_path
+
     @staticmethod
     def get_env_from_branch(branch_name):
         """Determine environment name from git branch name."""
@@ -512,7 +565,8 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
     @staticmethod
     def ensure_file_is_executable(path):
         """Exit if file is not executable."""
-        if not stat.S_IXUSR & os.stat(path)[stat.ST_MODE]:
+        if platform.system() != 'Windows' and (
+                not stat.S_IXUSR & os.stat(path)[stat.ST_MODE]):
             print("Error: File %s is not executable" % path)
             sys.exit(1)
 
@@ -531,58 +585,6 @@ class Base(object):  # noqa pylint: disable=too-many-instance-attributes,too-man
         finally:
             os.chdir(prevdir)
 
-    @staticmethod
-    @contextmanager
-    def override_sysargv(new_args=None):
-        """Temporarily spoof the arguments provided in calling the application.
-
-        Some applications will only detect their configuration from the command
-        line. A list provided to this method will override what they detect.
-
-        """
-        if new_args is None:
-            new_args = []
-        orig_args = sys.argv
-        sys.argv = new_args
-        try:
-            yield
-        finally:
-            sys.argv = orig_args
-
-    @staticmethod
-    @contextmanager
-    def turn_down_stacker_logging(command):
-        """Disable duplicate Stacker logging."""
-        stacker_loggers = [
-            'runway.embedded.stacker.actions.diff',
-            'runway.embedded.stacker.commands.stacker',
-            'runway.embedded.stacker.plan'
-        ]
-        if command == 'diff':
-            try:
-                for i in stacker_loggers:
-                    logging.getLogger(i).setLevel(logging.ERROR)
-                yield
-            finally:
-                for i in stacker_loggers:
-                    logging.getLogger(i).setLevel(logging.INFO)
-        else:
-            yield
-
-    @staticmethod
-    @contextmanager
-    def use_embedded_pkgs():
-        """Temporarily prepend embedded packages to sys.path."""
-        old_sys_path = sys.path
-        sys.path.insert(
-            1,  # https://stackoverflow.com/a/10097543
-            EMBEDDED_LIB_PATH
-        )
-        try:
-            yield
-        finally:
-            sys.path = old_sys_path
-
     @staticmethod
     @contextmanager
     def ignore_exit_code_0():
