@@ -23,11 +23,36 @@ from .base import Base
 # from util but that would require sys.path shenanigans here
 from ..embedded.stacker.awscli_yamlhelper import yaml_parse as parse_cloudformation_template  # noqa
 
-logging.basicConfig(level=logging.INFO)
-logging.getLogger('botocore').setLevel(logging.ERROR)  # their info is spammy
 LOGGER = logging.getLogger('runway')
 
 
+def make_stacker_cmd_string(args, lib_path):
+    """Generate stacker invocation script from command line arg list.
+
+    This is the standard stacker invocation script, with the following changes:
+    * Adding our explicit arguments to parse_args (instead of leaving it empty)
+    * Overriding sys.argv
+    * Adding embedded runway lib directory to sys.path
+    """
+    return ("import sys;"
+            "sys.argv = ['stacker'] + {args};"
+            "sys.path.insert(1, '{lib_path}');"
+            "from stacker.logger import setup_logging;"
+            "from stacker.commands import Stacker;"
+            "stacker = Stacker(setup_logging=setup_logging);"
+            "args = stacker.parse_args({args});"
+            "stacker.configure(args);args.run(args)".format(args=str(args),
+                                                            lib_path=lib_path))
+
+
+def run_module_command(cmd_list, env_vars):
+    """Shell out to provisioner command."""
+    try:
+        subprocess.check_call(cmd_list, env=env_vars)
+    except subprocess.CalledProcessError as shelloutexc:
+        sys.exit(shelloutexc.returncode)
+
+
 class Module(Base):  # noqa pylint: disable=too-many-public-methods
     """Module deployment class."""
 
@@ -164,7 +189,7 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
     def run_serverless(self, environment, region, command='deploy'):
         """Run Serverless."""
         response = {'skipped_configs': False}
-        sls_cmd = ['npm', 'run-script', 'sls', '--', command]
+        sls_opts = [command]
 
         if not self.which('npm'):
             LOGGER.error('"npm" not found in path or is not executable; '
@@ -172,38 +197,57 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
             sys.exit(1)
 
         if 'CI' in self.env_vars and command != 'remove':
-            sls_cmd.append('--conceal')  # Hide secrets from serverless output
+            sls_opts.append('--conceal')  # Hide secrets from serverless output
 
         if 'DEBUG' in self.env_vars:
-            sls_cmd.append('-v')  # Increase logging if requested
+            sls_opts.append('-v')  # Increase logging if requested
 
-        sls_cmd.extend(['-r', region])
+        sls_opts.extend(['-r', region])
+        sls_opts.extend(['--stage', environment])
         sls_env_file = self.get_sls_config_file(environment, region)
-        sls_cmd.extend(['--stage', environment])
+
+        if self.which('npx'):
+            # Use npx if available (npm v5.2+)
+            LOGGER.debug('Using npx to invoke sls.')
+            # The nested sls-through-npx-via-subprocess command invocation
+            # requires this redundant quoting
+            sls_cmd = ['npx', '-c', "''sls %s''" % ' '.join(sls_opts)]
+        else:
+            LOGGER.debug('npx not found; falling back invoking sls shell '
+                         'script directly.')
+            sls_cmd = [
+                os.path.join(self.module_root,
+                             'node_modules',
+                             '.bin',
+                             'sls')
+            ] + sls_opts
 
         if os.path.isfile(os.path.join(self.module_root, sls_env_file)):
             if os.path.isfile(os.path.join(self.module_root, 'package.json')):
                 with self.change_dir(self.module_root):
-                    # Use npm ci if available (npm v5.7+)
-                    if self.use_npm_ci():
-                        LOGGER.info("Running npm ci on %s...",
-                                    os.path.basename(self.module_root))
-                        subprocess.check_call(['npm', 'ci'])
-                    else:
-                        LOGGER.info("Running npm install on %s...",
-                                    os.path.basename(self.module_root))
-                        subprocess.check_call(['npm', 'install'])
+                    if not self.deploy_opts.get('skip-npm-ci'):
+                        # Use npm ci if available (npm v5.7+)
+                        if self.use_npm_ci():
+                            LOGGER.info("Running npm ci on %s...",
+                                        os.path.basename(self.module_root))
+                            subprocess.check_call(['npm', 'ci'])
+                        else:
+                            LOGGER.info("Running npm install on %s...",
+                                        os.path.basename(self.module_root))
+                            subprocess.check_call(['npm', 'install'])
                     LOGGER.info("Running sls %s on %s (\"%s\")",
                                 command,
                                 os.path.basename(self.module_root),
-                                " ".join(sls_cmd))
-                    subprocess.check_call(sls_cmd, env=self.env_vars)
+                                # Strip out redundant npx quotes not needed
+                                # when executing the command directly
+                                " ".join(sls_cmd).replace('\'\'', '\''))
+                    run_module_command(cmd_list=sls_cmd,
+                                       env_vars=self.env_vars)
             else:
                 LOGGER.warn(
                     "Skipping serverless %s of %s; no \"package.json\" "
                     "file was found (need a package file specifying "
-                    "serverless in devDependencies & a \"sls\" script "
-                    "invoking \"sls\")",
+                    "serverless in devDependencies)",
                     command,
                     os.path.basename(self.module_root))
         else:
@@ -263,9 +307,9 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
                         LOGGER.info('Using backend config file %s',
                                     backend_tfvars_file)
                         self.remove_stale_tf_config(backend_tfvars_file)
-                        subprocess.check_call(
-                            init_cmd + ['-backend-config=%s' % backend_tfvars_file],  # noqa
-                            env=self.env_vars
+                        run_module_command(
+                            cmd_list=init_cmd + ['-backend-config=%s' % backend_tfvars_file],  # noqa pylint: disable=line-too-long
+                            env_vars=self.env_vars
                         )
                     else:
                         LOGGER.info(
@@ -275,7 +319,8 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
                             ', '.join(self.gen_backend_tfvars_files(
                                 environment,
                                 region)))
-                        subprocess.check_call(init_cmd, env=self.env_vars)
+                        run_module_command(cmd_list=init_cmd,
+                                           env_vars=self.env_vars)
                 LOGGER.debug('Checking current Terraform workspace...')
                 current_tf_workspace = subprocess.check_output(
                     ['terraform',
@@ -296,27 +341,27 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
                     )
                     if re.compile("^[*\\s]\\s%s$" % environment,
                                   re.M).search(available_tf_envs):
-                        subprocess.check_call(
-                            ['terraform',
-                             'workspace',
-                             'select',
-                             environment],
-                            env=self.env_vars)
+                        run_module_command(
+                            cmd_list=['terraform', 'workspace', 'select',
+                                      environment],
+                            env_vars=self.env_vars
+                        )
                     else:
                         LOGGER.info("Terraform workspace %s not found; "
                                     "creating it...",
                                     environment)
-                        subprocess.check_call(
-                            ['terraform',
-                             'workspace',
-                             'new',
-                             environment],
-                            env=self.env_vars)
+                        run_module_command(
+                            cmd_list=['terraform', 'workspace', 'new',
+                                      environment],
+                            env_vars=self.env_vars
+                        )
                 if 'SKIP_TF_GET' not in self.env_vars:
                     LOGGER.info('Executing "terraform get" to update remote '
                                 'modules')
-                    subprocess.check_call(['terraform', 'get', '-update=true'],
-                                          env=self.env_vars)
+                    run_module_command(
+                        cmd_list=['terraform', 'get', '-update=true'],
+                        env_vars=self.env_vars
+                    )
                 else:
                     LOGGER.info('Skipping "terraform get" due to '
                                 '"SKIP_TF_GET" environment variable...')
@@ -324,7 +369,7 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
                             command,
                             os.path.basename(self.module_root),
                             " ".join(tf_cmd))
-                subprocess.check_call(tf_cmd, env=self.env_vars)
+                run_module_command(cmd_list=tf_cmd, env_vars=self.env_vars)
         else:
             response['skipped_configs'] = True
             LOGGER.info("Skipping Terraform %s of %s",
@@ -359,12 +404,17 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
 
         with self.change_dir(self.module_root):
             # Iterate through any stacker yaml configs to deploy them in order
+            # or destroy them in reverse order
             for _root, _dirs, files in os.walk(self.module_root):
-                for name in sorted(files):
-                    if name == 'runway.yml':
+                for name in (
+                        reversed(sorted(files))
+                        if command == 'destroy'
+                        else sorted(files)):
+                    if name == 'runway.yml' or name.startswith('.'):
+                        # Hidden files (e.g. .gitlab-ci.yml) or runway configs
+                        # definitely aren't stacker config files
                         continue
-                    file_extension = os.path.splitext(name)[1]
-                    if file_extension in ['.yaml', '.yml']:
+                    if os.path.splitext(name)[1] in ['.yaml', '.yml']:
                         if not os.path.isfile(os.path.join(self.module_root,
                                                            stacker_env_file)):
                             response['skipped_configs'] = True
@@ -385,52 +435,54 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
                         LOGGER.info("Running stacker %s on %s",
                                     command,
                                     name)
-                        # Need to override command line arguments (even when
-                        # running stacker directly like this) because its
-                        # hooks may call util.get_config_directory() which
-                        # checks the command line arguments directly
-                        with self.override_sysargv(['stacker'] + stacker_cmd):
-                            # Ensure any blueprints use our embedded version of
-                            # stacker
-                            with self.use_embedded_pkgs():
-                                with self.turn_down_stacker_logging(command):
-                                    with self.override_env_vars():
-                                        from ..embedded.stacker.commands import Stacker  # noqa
-                                        stacker = Stacker()
-                                        args = stacker.parse_args(
-                                            stacker_cmd + [name]
-                                        )
-                                        stacker.configure(args)
-                                        args.run(args)
+                        stacker_cmd_str = make_stacker_cmd_string(
+                            stacker_cmd + [name],
+                            self.embedded_lib_path
+                        )
+                        stacker_cmd_list = [sys.executable, '-c']
+                        LOGGER.debug(
+                            "Stacker command being executed: %s \"%s\"",
+                            ' '.join(stacker_cmd_list),
+                            stacker_cmd_str
+                        )
+                        run_module_command(
+                            cmd_list=stacker_cmd_list + [stacker_cmd_str],
+                            env_vars=self.env_vars
+                        )
                 break  # only need top level files
         return response
 
     def determine_module_type(self):
         """Determine type of module."""
+        module_type = ''
         # First check directory name for type-indicating suffix
         if os.path.basename(self.module_root).endswith('.sls'):
-            return 'serverless'
+            module_type = 'serverless'
         elif os.path.basename(self.module_root).endswith('.tf'):
-            return 'terraform'
+            module_type = 'terraform'
         elif os.path.basename(self.module_root).endswith('.cfn'):
-            return 'stacker'
+            module_type = 'stacker'
         # Fallback to autodetection
-        if os.path.isfile(os.path.join(self.module_root, 'serverless.yml')):
-            return 'serverless'
-        elif glob.glob(os.path.join(self.module_root, '*.tf')):
-            return 'terraform'
-        elif glob.glob(os.path.join(self.module_root, '*.env')):
-            return 'stacker'
-        else:
+        if module_type == '':
+            if os.path.isfile(os.path.join(self.module_root,
+                                           'serverless.yml')):
+                module_type = 'serverless'
+            elif glob.glob(os.path.join(self.module_root, '*.tf')):
+                module_type = 'terraform'
+            elif glob.glob(os.path.join(self.module_root, '*.env')):
+                module_type = 'stacker'
+        if module_type == '':
             LOGGER.error('No valid deployment configurations found for %s',
                          os.path.basename(self.module_root))
             sys.exit(1)
+        return module_type
 
     def get_and_update_region(self):
         """Find AWS region, prompting if necessary."""
         if 'AWS_DEFAULT_REGION' not in self.env_vars:
             aws_region = input("Please enter the AWS region: ")
-            self.update_env_vars({'AWS_DEFAULT_REGION': aws_region})
+            self.update_env_vars({'AWS_DEFAULT_REGION': aws_region,
+                                  'AWS_REGION': aws_region})
         return self.env_vars['AWS_DEFAULT_REGION']
 
     def plan(self):
@@ -612,8 +664,16 @@ class Module(Base):  # noqa pylint: disable=too-many-public-methods
         names = []
         for ext in ['yml', 'json']:
             # Give preference to explicit stage-region files
+            names.append(
+                os.path.join('env',
+                             "%s-%s.%s" % (stage, region, ext))
+            )
             names.append("config-%s-%s.%s" % (stage, region, ext))
             # Fallback to stage name only
+            names.append(
+                os.path.join('env',
+                             "%s.%s" % (stage, ext))
+            )
             names.append("config-%s.%s" % (stage, ext))
         return names
 
