@@ -16,8 +16,10 @@ class Action(BaseAction):
     def run(self, *args, **kwargs):
         logger.info('Outputs for stacks: %s', self.context.get_fqn())
         for stack in self.context.get_stacks():
+            provider = self.build_provider(stack)
+
             try:
-                provider_stack = self.provider.get_stack(stack.fqn)
+                provider_stack = provider.get_stack(stack.fqn)
             except exceptions.StackDoesNotExist:
                 logger.info('Stack "%s" does not exist.' % (stack.fqn,))
                 continue
