@@ -28,7 +28,8 @@ class Diff(BaseCommand):
 
     def run(self, options, **kwargs):
         super(Diff, self).run(options, **kwargs)
-        action = diff.Action(options.context, provider=options.provider)
+        action = diff.Action(options.context,
+                             provider_builder=options.provider_builder)
         action.execute()
 
     def get_context_kwargs(self, options, **kwargs):
