@@ -1,5 +1,6 @@
 import threading
 import logging
+from getpass import getpass
 
 
 logger = logging.getLogger(__name__)
@@ -48,6 +49,14 @@ class UI(object):
         finally:
             self.unlock()
 
+    def getpass(self, *args):
+        """Wraps getpass to lock the UI."""
+        try:
+            self.lock()
+            return getpass(*args)
+        finally:
+            self.unlock()
+
 
 # Global UI object for other modules to use.
 ui = UI()
